"""
    This file is part of Accountant.

    Accountant is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Accountant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with Accountant.  If not, see <http://www.gnu.org/licenses/>.
"""
# vim: set tw=80 ts=4 sw=4 sts=4:
import arrow

from functools import wraps

from flask import request, g
from flask.ext.restplus import Resource, marshal_with_field

from accountant import app

from .. import api

from ..fields import Object

from ..models.users import User

from .models import user_model, token_model, login_model


def load_user_from_token(token):
    return User.verify_auth_token(token)


def load_user_from_auth(auth):
    token = auth.replace('Bearer ', '', 1)
    return load_user_from_token(token)


def requires_auth(f):
    @wraps(f)
    def wrapped(*args, **data):
        user = None

        if 'Authorization' in request.headers:
            auth = request.headers['Authorization']
            user = load_user_from_auth(auth)

        if user:
            g.user = user
            return f(*args, **data)

        api.abort(
            401,
            error_message='Please login before executing this request.'
        )
    return wrapped


ns = api.namespace('user', description='User management')


@ns.route('/login')
class LoginResource(Resource):
    @api.marshal_with(token_model)
    @api.doc(
        responses={
            200: ('OK', token_model),
            401: 'Unauthorized'
        })
    @api.expect(login_model)
    def post(self):
        """
        Login to retrieve authentication token.
        """
        data = api.payload

        user = User.query().filter(
            User.email == data['email']
        ).one_or_none()

        if not user or not user.verify_password(data['password']):
            api.abort(401, error_message="Bad user or password.")

        token = user.generate_auth_token()
        expiration_time = arrow.now().replace(
            seconds=app.config['SESSION_TTL']
        )

        return {
            'token': token,
            'expiration': expiration_time.datetime,
            'token_type': 'Bearer'
        }, 200

    @requires_auth
    @api.doc(
        security='apikey',
        responses={
            200: ('OK', user_model)
        })
    @marshal_with_field(Object(user_model))
    def get(self):
        """
        Get authenticated user information.
        """
        return g.user, 200
